<?php

class Reports_Model_WorldWeatherOnlineWeatherService extends Reports_Model_WeatherServiceAbstract {
    protected $_host = 'www.worldweatheronline.com';
    static $_keepGoing = True;
    public $requests = array('from cache' => array('nbr' => 0, 'responds' => array('errors'=>array()), 'urls' => array(), 'errorurls' => array()),
                             'from web'  => array('nbr' => 0, 'responds' => array('errors'=>array(),'http_status'=>array()), 'urls' => array(), 'errorurls' => array(), 'curlurls' => array()));

    /*Fake API data. Can be used for testing purposes:
     * MOVED TO WIKI
     */


    /*
     * get weather data at three hour intervals for $date.
     */
    public function getDayWeather($date, $position) {
        $ret = array();
        try {
            $day_weather = $this->_get_weather($date, $position);
        } catch(WebReport_Exception $e) {
            foreach(range(1,7) as $r) {
                $ret[] = new Reports_Model_WeatherData(0, 0, 0, $e->getCode(), false, $e->getMessage());
            }
            $this->log($this->requests);
            return $ret;
        }
        foreach($day_weather['data']['weather'][0]['hourly'] as $hour) {
            $time = $this->_mil_to_norm($hour['time']);
            $ret[] = new Reports_Model_WeatherData($time, $hour['tempC'], $hour['precipMM']/3, $hour['weatherCode']);
        }
        $this->log($this->requests);
        return $ret;
    }

    public function getWeekWeather($date, $position) {
        $ret = array();
        $weekdays = array("SUNDAY", "MONDAY", "TUESDAY", "WEDNESDAY", "THURSDAY", "FRIDAY", "SATURDAY");

//        $translate = Zend_Registry::get("Zend_Translate");
//        foreach($weekdays as $i=>$day)
//            $weekdays[$i] = $translate->translate($day);

        $now = new Zend_Date();

        $week = $date->get(Zend_Date::WEEK);
        for($i = 0; $i < 8; $i++){
            $date = $date->sub(1, Zend_Date::DAY);
            if($date->get(Zend_Date::WEEK) != $week) break;
        }


        if(CM_Interval::firstDayOfWeek() == "monday"){
            $date->add(1,Zend_Date::DAY);
        }

        for($i = 0; $i < 7; $i++){
            $weekday = $date->get(Zend_Date::WEEKDAY_DIGIT);
            if($date->compare($now) > 0) {
                $day = new Reports_Model_WeatherData($weekdays[$weekday], 0, 0, 3, false); //Create with future request error. No error message is needed, this is normal.
            } else {
                try {
                    $day_weather_array = $this->_get_weather($date, $position, true);
                } catch(WebReport_Exception $e) {
                    $ret[] = new Reports_Model_WeatherData(0, 0, 0, $e->getCode(), false, $e->getMessage());
                    continue;
                }
                $hour = $day_weather_array['data']['weather'][0]['hourly'][0];
                $day = new Reports_Model_WeatherData($weekdays[$weekday], $hour['tempC'], $hour['precipMM']/24, $hour['weatherCode']);
            }

            $ret[] = $day;
            $date->add(1,Zend_Date::DAY);
        }

        $this->log($this->requests);
        return $ret;
    }

    /*
     * convert military time (e.g. 400) to normal format (04:00)
     */
    protected function _mil_to_norm($hour) {
        $min = $hour%100;
        $hour = (int) floor($hour/100);

        return sprintf("%02s:%02s", $hour, $min);
    }

    protected function _get_weather($date, $position, $whole_day = false) {
        $cache = 0;
        $tmp = new Zend_Date($date);

        if(abs($tmp->sub(new Zend_Date())->toValue()) < 86400) {
            $cache = Zend_Registry::get('short-weather-cache');
        } else  {
            $cache = Zend_Registry::get('weather-cache');
        }

        $request_url = $this->_get_request_url($date,$position, $whole_day);
        $key = preg_replace('/[^a-zA-Z0-9_]/', '', $request_url);

        $result = "";

        if($cache && ($val = $cache->load($key))) {
            $result = $val;
            $weather_data = json_decode($result, true);

            $this->requests['from cache']['nbr'] ++;
            try{
                $this->_check_respond($weather_data);
            }catch(WebReport_Exception $e){
                $this->requests['from cache']['responds']['errors'][]=$e->getMessage();
                $this->requests['from cache']['errorurls'][] = $request_url;
                throw new WebReport_Exception_WeatherError("Error retrieving weather data from cache: " . $e->getMessage(), 1);
            }
            $this->requests['from cache']['urls'][] = $request_url;
        }else {
            if(self::$_keepGoing == false){
				throw new WebReport_Exception_WeatherError("Weather curl request failed its three attempts", 0);
            }
            foreach(range(1,3) as $i){
                if($result = $this->_curla($request_url)){
                    if($weather_data = json_decode($result, true)){
                        if(!isset($weather_data['data']['error']) & isset($weather_data['data']['weather'][0]['hourly'])){
                            break;
                        }
                    }
                }
                sleep(1);
                error_log("Weather curl request failed, sleeping..");
                if($i == 3){
                    self::$_keepGoing = false;
                }
            }
            if(!$result){
                throw new WebReport_Exception_WeatherError("No data from host. Possible timeout.",0);
                $this->requests['from web']['errorurls'][] = $request_url;
            }
            try{
                $this->_check_respond($weather_data);
            }catch(WebReport_Exception $e){
                $this->requests['from web']['responds']['errors'][]=$e->getMessage();
                $this->requests['from web']['errorurls'][] = $request_url;
                throw new WebReport_Exception_WeatherError("No data recived from" . $this->_host . ": " .  $e->getMessage(), 0);
            }
        }
        if( $cache ) $cache->save($result);
        return $weather_data;
    }

    protected function _curla($request_url){
        // get the weather data with curl
        $ch = curl_init($request_url);
        $options = array(
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_HTTPHEADER => array('Content-type: application/json'),
            CURLOPT_TIMEOUT => 10
            );
        curl_setopt_array($ch, $options);
        $res = curl_exec($ch);
        $http_status = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);
        $this->requests['from web']['nbr'] ++;
        $this->requests['from web']['responds']['http_status'][] = $http_status;
        $this->requests['from web']['curlurls'][] = $request_url;
        return $res;
    }

    public function _check_respond($weather_data){
        if(!$weather_data){
            throw new WebReport_Exception_WeatherError("Weather data object not initialized. json decode fail?",5);
        }
        if(isset($weather_data['data']['error'])) {
            $msg = $weather_data['data']['error'][0]['msg'];
            throw new WebReport_Exception_WeatherError("Server reported an error: " . $msg, 2);
        }
        if(!isset($weather_data['data']['weather'][0]['hourly'])) {
            throw new WebReport_Exception_WeatherError('No weather data found on server for requested day.',4);
        }
    }

    protected function _get_request_url($date, $position, $whole_day = false)
    {
        $keys = $this->_getKeys();

        if(count($keys == 1)) {
            $base = "http://api.worldweatheronline.com/premium/v1/past-weather.ashx?q=%s&format=json&key=%s&date=%s%s";

            $url = sprintf($base,
                    $position,
                    $keys[0],
                    $date->get('yyyy-MM-dd'),
                    $whole_day ? '&tp=24' : '');

            return $url;
        }

        if(count($keys) != 2) throw new WebReport_Exception("Incorrect Weather Settings. Need two keys when using World Weather Online.");

        //Nedan använd inte (?):
        $base = "http://%s/feed/premium-weather-v2.ashx?key=%s&feedkey=%s&format=json&q=%s&date=%s%s";
        if($this->_forcedSource()){
            $base = "http://%s/w?q=%s&date=%s%s";
            $host = $this->_forcedSource();
            $url  = sprintf($base,
                    $host,
                    $position,
                    $date->get('yyyy-MM-dd'),
                    $whole_day ? '&tp=24' : '');
            return $url;
        }

        $host = $this->_host;

        $url  = sprintf($base,
                $host,
                $keys[0], $keys[1],
                $position,
                $date->get('yyyy-MM-dd'),
                $whole_day ? '&tp=24' : '');
        return $url;
    }

    public function log($req){
        $logger = Zend_Registry::get('loggers')->getLogger('weather');

        $http_status_web = $req['from web']['responds']['http_status'];
        $errors_web = $req['from web']['responds']['errors'];
        $errors_cache = $req['from cache']['responds']['errors'];

        if ($req['from web']['nbr'] != 0){
            $logger->warn($req['from web']['nbr'] . ' weather requests to the web was made');
            if(!empty($http_status_web)){
                for($i=0; $i < count($http_status_web); $i++){
                    if($http_status_web[$i]==0){
                        $logger->err('Weather request to url: ' . $this->_keyless_url($req['from web']['curlurls'][$i]) . ' failed. Could not connect, aborting further request after three tries.');
                    }elseif($http_status_web[$i]!=200) {
                        $logger->err('Weather request to url: ' . $this->_keyless_url($req['from web']['curlurls'][$i]) . ' failed. HTTP response was:  ' . $http_status_web[$i]);
                    }
                }
            }
            if(!empty($errors_web)) {
                for($i=0; $i < count($errors_web); $i++){
                    $logger->err('Weather request to url: ' . $this->_keyless_url($req['from web']['errorurls'][$i]) . ' failed. Error message: ' . $errors_web[$i]);
                }
            }
        }

        if($req['from cache']['nbr']!=0){
            $logger->notice($req['from cache']['nbr'] . ' weather requests to the cache was made');
            if(!empty($errors_cache)) {
                for($i=0; $i < count($errors_cache); $i++){
                    $logger->err('Weather request to cache with key: ' . $this->_keyless_url($req['from cache']['errorurls'][$i]) . ' failed. Error message: ' . $errors_cache[$i]);
                }
            }
        }
    }

     public function testService()
    {
        $this->_get_weather(new Zend_Date(), "Lund");
    }
    public function _keyless_url($url){
    	$urlFirst = explode("key=", $url, 2);
    	$urlLast = explode("&date=", $url, 2);
    	$res = $urlFirst[0] . "key=[HIDDEN]&date=" . $urlLast[1];
    	return $res;
    }

}
