<?php

class Sets_Model_Api_Method_Sets_GetLocations extends WebReport_Api_Method_Abstract
{
    public function userIsAllowed(Core_Model_User $user)
    {
        return true;
    }

    public function needsAuthentication()
    {
        /*
		  if($_SERVER['REMOTE_ADDR'] == "127.0.0.1" ||
		  $_SERVER['REMOTE_ADDR'] == "::1" ) {
		  return false;
		  }
        */
        return false;
    }

    protected function _get_request_url($p) {
        $config = Zend_Registry::get('config');
        $base_url = $config->general->weatherApiUrl;
        $api_key = $config->general->weatherApiKey;

        // Todo add/remove ?
        // Todo add remove protocol

        $endpoint = $base_url . '?key=' . $api_key;
        foreach ($p as $key => $value) {
            $endpoint .= '&' . $key . '=' . urlencode($value);
        }
        return $endpoint;
    }

    protected function set_api_active($value) {
        $config = Zend_Registry::get('config');
        $writer = new Zend_Config_Writer_IniNice();
        $writer->setConfig($config)->setFilename(APPLICATION_PATH . '/config.ini');
        $config->general->weatherApiEnabled = $value;
        try {
            $writer->write();
            return "ok";
        } catch (Zend_Config_Exception $e) {
            return "Failed to write to config.ini file";
        }
    }

    protected function api_is_active() {
        $config = Zend_Registry::get('config');
        return $config->general->weatherApiEnabled;
    }

    public function call(array $params)
    {
        if (array_key_exists('active', $params['params'])) {
            $val = $params['params']['active'];
            if ($val == 'false' || $val == 'true')
                echo json_encode(array("status" => $this->set_api_active($val=='true'?'1':'0')));
            elseif ($val == '?')
                echo json_encode(array("status" => "ok", "value"=>$this->api_is_active()));
            else
                echo json_encode(array("status" => "failed", "msg" => "Unknown query"));
            return;
        }
        if (!array_key_exists('city', $params['params'])) {
            r_log('Received getLocation request without specified city...');
            return;
        }

        $config = Zend_Registry::get('config');
        if (!$config->general->weatherApiEnabled || 
            strlen($config->general->weatherApiKey) == 0 ||
            strlen($config->general->weatherApiUrl) == 0) {
            echo json_encode(array(
                "status" => "failed", 
                "msg" => "LOCATION_API_NOT_ACTIVE", 
                "query" => $params['params']['city']));
            return;
        }

        // Build the get string..
        $p = array(
            'query' => $params['params']['city'],
            'format' => 'json'
        );
        $request_url = $this->_get_request_url($p);

        // Build options for curl
        $options = array(
                         CURLOPT_RETURNTRANSFER => true,
                         CURLOPT_HTTPHEADER => array('Content-type: application/json'),
                         CURLOPT_SSL_VERIFYPEER => false,
                         CURLOPT_SSL_VERIFYHOST => false
                       );

        if (!empty($config->general->httpProxy))
          $options[CURLOPT_PROXY] = $config->general->httpProxy;
        if (!empty($config->general->httpProxyPort))
          $options[CURLOPT_PROXYPORT] = $config->general->httpProxyPort;
        if (!empty($config->general->httpProxyUser) && !empty($config->general->httpProxyPass))
          $options[CURLOPT_PROXYUSERPWD] = $config->general->httpProxyUser . ":" . $config->general->httpProxyPass;

        // Make curl request
        $ch = curl_init($request_url);
        curl_setopt_array($ch, $options);
        $result = curl_exec($ch);
        if ($result === false) {
            echo json_encode(array("status" => "failed", "msg" => "QUERY_FAILED"));
            return;
        }
        echo $result;
    }
}
